'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs = require('rxjs');
var operators = require('rxjs/operators');
var env = require('../../common/env.js');

const defaultSerializer = data => JSON.stringify(data);

const defaultDeserializer = message => message.data;

const isMessageEvent = event => event.type === 'message';

class RxWebSocket {
  websocket;
  opened$;
  events$ = new rxjs.Subject();
  serializer = defaultSerializer;
  deserializer = defaultDeserializer;
  debug;

  constructor({
    url,
    protocols,
    options,
    serializer = defaultSerializer,
    deserializer = defaultDeserializer,
    debug = false
  }) {
    const websocket = env.isBrowser() ? new WebSocket(url, protocols) : new (require('ws'))(url, protocols, options);
    const close$ = rxjs.fromEvent(websocket, 'close').pipe(rxjs.take(1), operators.share());
    const open$ = rxjs.fromEvent(websocket, 'open').pipe(operators.takeUntil(close$), rxjs.shareReplay(1), rxjs.take(1));
    const message$ = rxjs.fromEvent(websocket, 'message').pipe(operators.takeUntil(close$), rxjs.shareReplay(1));
    const error$ = rxjs.fromEvent(websocket, 'error').pipe(operators.takeUntil(close$), rxjs.mergeMap(rxjs.throwError));
    rxjs.merge(open$, message$, error$, close$).subscribe(this.events$);
    this.websocket = websocket;
    this.opened$ = open$;
    this.serializer = serializer.bind(this);
    this.deserializer = deserializer.bind(this);
    this.debug = debug;

    if (this.debug) {
      this.events$.subscribe(ev => console.log(`[websocket] message: type=${ev.type}`, 'data' in ev ? ev.data : undefined));
    }
  }

  get active() {
    return !this.events$.closed;
  }

  get message() {
    return this.events$.asObservable().pipe(rxjs.filter(isMessageEvent), operators.map(this.deserializer));
  }

  send(data) {
    return this.opened$.pipe(rxjs.mergeMap(() => {
      const message = this.serializer(data);

      if (this.debug) {
        console.log('[websocket] send:', message);
      }

      this.websocket.send(message);
      return this.message;
    }));
  }

}

exports.RxWebSocket = RxWebSocket;
//# sourceMappingURL=websocket.js.map
