import { Subject, fromEvent, take, shareReplay, mergeMap, throwError, merge, filter } from 'rxjs';
import { share, takeUntil, map } from 'rxjs/operators';
import { isBrowser } from '../../common/env.js';

const defaultSerializer = (data) => JSON.stringify(data);
const defaultDeserializer = (message) => message.data;
const isMessageEvent = (event) => event.type === 'message';
class RxWebSocket {
    websocket;
    opened$;
    events$ = new Subject();
    serializer = defaultSerializer;
    deserializer = defaultDeserializer;
    debug;
    constructor({ url, protocols, options, serializer = defaultSerializer, deserializer = defaultDeserializer, debug = false, }) {
        const websocket = isBrowser() ? new WebSocket(url, protocols) : new (require('ws'))(url, protocols, options);
        const close$ = fromEvent(websocket, 'close').pipe(take(1), share());
        const open$ = fromEvent(websocket, 'open').pipe(takeUntil(close$), shareReplay(1), take(1));
        const message$ = fromEvent(websocket, 'message').pipe(takeUntil(close$), shareReplay(1));
        const error$ = fromEvent(websocket, 'error').pipe(takeUntil(close$), mergeMap(throwError));
        merge(open$, message$, error$, close$).subscribe(this.events$);
        this.websocket = websocket;
        this.opened$ = open$;
        this.serializer = serializer.bind(this);
        this.deserializer = deserializer.bind(this);
        this.debug = debug;
        if (this.debug) {
            this.events$.subscribe((ev) => console.log(`[websocket] message: type=${ev.type}`, 'data' in ev ? ev.data : undefined));
        }
    }
    get active() {
        return !this.events$.closed;
    }
    get message() {
        return this.events$.asObservable().pipe(filter(isMessageEvent), map(this.deserializer));
    }
    send(data) {
        return this.opened$.pipe(mergeMap(() => {
            const message = this.serializer(data);
            if (this.debug) {
                console.log('[websocket] send:', message);
            }
            this.websocket.send(message);
            return this.message;
        }));
    }
}

export { RxWebSocket };
//# sourceMappingURL=websocket.js.map
