import { BehaviorSubject, firstValueFrom, mergeMap, interval } from 'rxjs';
import { first, timeout, takeUntil } from 'rxjs/operators';
import { CONFIG, packetTypes } from './constants.js';
import { toSocketIoMessagePayload, isResponseMessageOf, scrapboxSerializer, scrapboxDeserializer, isConnectionMessage, isResponseOf } from './internal/message.js';
import { createChanges } from './internal/request.js';
import { isNotNull, getAuthCookieValue } from './internal/util.js';
import { RxWebSocket } from './internal/websocket.js';

class ScrapboxWebsocketHandler {
    options;
    socket$ = new BehaviorSubject(null);
    room = null;
    sid = 0;
    constructor(options) {
        this.options = options;
    }
    async commit({ projectId, userId, pageId, parentId, changes, }) {
        await this.join({ projectId, pageId });
        return this.send({
            projectId,
            pageId,
            data: {
                method: 'commit',
                data: { kind: 'page', userId, projectId, pageId, parentId, changes: createChanges(changes, userId), cursor: null, freeze: true },
            },
        });
    }
    async join({ projectId, pageId }) {
        // join room if needed
        if (this.room?.projectId === projectId && this.room?.pageId === pageId) {
            return;
        }
        await this.send({
            projectId,
            pageId,
            data: {
                method: 'room:join',
                data: { pageId, projectId, projectUpdatesStream: false },
            },
        });
        this.room = { projectId, pageId };
    }
    send({ data, projectId, pageId }) {
        const sid = String(this.sid++);
        return firstValueFrom(this.getSocket().pipe(first(isNotNull), mergeMap((socket) => socket.send(toSocketIoMessagePayload(sid, data)).pipe(first(isResponseMessageOf(sid)), timeout(CONFIG.responseTimeout)))));
    }
    getSocket() {
        if (!this.socket$.getValue()?.active) {
            return this.openNewSocket();
        }
        return this.socket$.asObservable();
    }
    openNewSocket() {
        const socket = new RxWebSocket({
            url: CONFIG.endpoint,
            options: { headers: { Origin: CONFIG.origin, Cookie: getAuthCookieValue(this.options.token ?? '') } },
            serializer: scrapboxSerializer,
            deserializer: scrapboxDeserializer,
            debug: this.options.debug,
        });
        return socket.message.pipe(first(isConnectionMessage), mergeMap(([, data]) => {
            return socket.send(packetTypes.connected).pipe(first(isResponseOf(packetTypes.connected)), mergeMap(() => {
                interval(data.pingInterval)
                    .pipe(takeUntil(socket.message))
                    .subscribe(() => socket.send(packetTypes.ping));
                this.socket$.next(socket);
                return this.socket$.asObservable();
            }));
        }));
    }
}

export { ScrapboxWebsocketHandler };
//# sourceMappingURL=websocket-handler.js.map
